/**************************************************************************
** Filename: Led.cpp
**
** Title: Series 5000 Eyetracker Host Software
**
** Description:
** This file contains routines that implement the Led component.
**
***************************************************************************
**
** Modification History:
**    1.0,  05/14/97, THC, Initial version.
**
**************************************************************************/

//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Led.h"
//---------------------------------------------------------------------------
static inline TLed *ValidCtrCheck()
{
  return new TLed(NULL);
}
//---------------------------------------------------------------------------
__fastcall TLed::TLed(TComponent* Owner)
  : TGraphicControl(Owner)
{
  ControlStyle << csOpaque << csSetCaption;
  FAlignment = laCenter;
  FBorderColor = clBlack;
  FBorderStyle = bsRaised;
  FBorderWidth = 2;
//  FCaption = "";
  FCtl3D = true;
  FFont = new TFont();
  FInterval = 0;
  FLedHeight = 18;
  FLedOn = false;
  FLedWidth = 18;
  FLedShape = lsRound;
  FMargin = 3;
  FOffColor = clMaroon;
  FOnColor = clRed;
  FPhongSize = 2;
  Height = 42;
  Width = 66;
}
//---------------------------------------------------------------------------

__fastcall TLed::~TLed( void )
{
  FTimer -> Free();
  FFont -> Free();
}


//---------------------------------------------------------------------------
namespace Led
{
  void __fastcall Register()
  {
    TComponentClass classes[1] = {__classid(TLed)};
    RegisterComponents("ASL", classes, 0);
  }
}
//---------------------------------------------------------------------------


void __fastcall TLed::SetAlignment( TMyAlignment value )
{
  if ( value != FAlignment )
    {
    FAlignment = value;
    Invalidate();
    }
}

void __fastcall TLed::SetBorderStyle( TBrdrStyle value )
{
  if ( value != FBorderStyle )
    {
    FBorderStyle = value;
    Invalidate();
    }
}

void __fastcall TLed::SetBorderWidth( TBrdrWidth value )
{
  if ( value != FBorderWidth )
    {
    FBorderWidth = value;
    Invalidate();
    }
}

void __fastcall TLed::SetCaption( TCaption value )
{
  if ( value != FCaption )
    {
    FCaption = value;
    Invalidate();
    }
}

void __fastcall TLed::SetCtl3D( bool value )
{
  if ( value != FCtl3D )
    {
    FCtl3D = value;
    Invalidate();
    }
}

#if 1
void __fastcall TLed::SetFont( TFont *value )
{
  if ( value != FFont )
    {
    FFont -> Assign( value );
    Invalidate();
    }
}
#endif

void __fastcall TLed::SetInterval( TInterval value )
{
  if ( value != FInterval )
    {
    FInterval = value;
    if ( FTimer ) FTimer -> Interval = FInterval;
    }
}

void __fastcall TLed::SetLedDimension( int index, TLedDimension value )
{
  switch ( index )
    {
    case 1 : if ( FLedHeight != value ) FLedHeight = value; break;
    case 2 : if ( FLedWidth != value ) FLedWidth = value; break;
    }
  if ( FLedHeight > Height ) Height = FLedHeight;
  if ( FLedWidth > Width ) Width = FLedWidth;
  Invalidate();
}

void __fastcall TLed::SetLedOn(bool value )
{
  if ( value != FLedOn )
    {
    FLedOn = value;
    if ( FOnOff ) FOnOff( this, FLedOn );
    Invalidate();
    }
}

void __fastcall TLed::SetLedShape( TLedShape value )
{
  if ( value != FLedShape )
    {
    FLedShape = value;
    Invalidate();
    }
}

void __fastcall TLed::SetColor( int index, TColor value )
{
  switch ( index )
    {
    case 1 : if ( value != FOffColor ) FOffColor = value; break;
    case 2 : if ( value != FOnColor ) FOnColor = value; break;
    case 3 : if ( value != FBorderColor ) FBorderColor = value; break;
    }
  Invalidate();
}

void __fastcall TLed::SetMargin( TMargin value )
{
  if ( value != FMargin )
    {
    FMargin = value;
    Invalidate();
    }
}

void __fastcall TLed::SetPhongSize( TBrdrWidth value )
{
  if ( value != FPhongSize )
    {
    FPhongSize = value;
    Invalidate();
    }
}

void __fastcall TLed::TimerTimer( TObject *Sender )
{
  if ( LedOn == true )
    LedOn = false;
  else
    LedOn = true;
}

void __fastcall TLed::Loaded( void )
{
  if ( ComponentState . Contains( csDesigning ) == false )
    {
    if ( FInterval > 0 )
      {
      FTimer = new TTimer( this );
      FTimer -> Interval = FInterval;
      FTimer -> OnTimer = TimerTimer;
      }
    }
}

void __fastcall TLed::Paint( void )
{
  // at design-time clean old layout
  if ( ComponentState . Contains( csDesigning ) )
    {
    Canvas -> Brush -> Style = bsSolid;
    Canvas -> Brush -> Color = Color;
    Canvas -> Pen -> Style = psDot;
    Canvas -> Pen -> Color = clGray;
    Canvas -> Rectangle(0, 0, Width, Height);
    }
  // setup brush
  if ( FLedOn )
    Canvas -> Brush -> Color = FOnColor;
  else
    Canvas -> Brush -> Color = FOffColor;
  Canvas -> Brush -> Style = bsSolid;
  // setup pen
  if ( ( Ctl3D == true ) && ( FLedShape == lsRect ) )
    Canvas -> Pen -> Color = Canvas -> Brush -> Color;
  else
    Canvas -> Pen -> Color = FBorderColor;
  Canvas -> Pen -> Style = psSolid;
  Canvas -> Pen -> Width = FBorderWidth;
  // setup font
  Canvas -> Font = FFont;
  // compute positions
  switch ( FAlignment )
    {
    case laBottom :
      LedOrigin . x = ( Width - FLedWidth ) / 2;
      LedOrigin . y = Height - FLedHeight;
      CaptionOrigin . x = ( Width - Canvas -> TextWidth( FCaption ) ) / 2;
      CaptionOrigin . y = LedOrigin.y - FMargin - Canvas -> TextHeight( FCaption );
      break;
    case laCenter :
      LedOrigin.x = ( Width - FLedWidth ) / 2;
      LedOrigin.y = ( Height - FLedHeight ) / 2;
      CaptionOrigin.x = ( Width - Canvas -> TextWidth( FCaption ) ) / 2;
      CaptionOrigin.y = LedOrigin.y + FLedHeight + FMargin;
      break;
    case laLeft  :
      LedOrigin.x = 1;
      LedOrigin.y = ( Height - FLedHeight ) / 2;
      CaptionOrigin.x = LedOrigin.x + FLedWidth + FMargin;
      CaptionOrigin.y = ( Height - Canvas -> TextHeight( FCaption ) ) / 2;
      break;
    case laRight :
      LedOrigin.x = Width - FLedWidth;
      LedOrigin.y = ( Height - FLedHeight ) / 2;
      CaptionOrigin.x = LedOrigin.x - FMargin - Canvas -> TextWidth( FCaption );
      CaptionOrigin.y = ( Height - Canvas -> TextHeight( FCaption ) ) / 2;
      break;
    case laTop   :
      LedOrigin.x = ( Width - FLedWidth ) / 2;
      LedOrigin.y = 1;
      CaptionOrigin.x = ( Width - Canvas -> TextWidth( FCaption ) ) / 2;
      CaptionOrigin.y = LedOrigin.y + FLedHeight + FMargin;
      break;
    }  // end switch
  // draw led
  LedRect = Rect( LedOrigin.x, LedOrigin.y, LedOrigin.x+FLedWidth, LedOrigin.y+FLedHeight );
  switch ( FLedShape )
    {
    case lsRect    :
      Canvas -> Rectangle(LedRect.Left+1, LedRect.Top+1, LedRect.Right-1, LedRect.Bottom-1);
      break;
    case lsRndRect :
      Canvas -> RoundRect(LedRect.Left+1, LedRect.Top+1, LedRect.Right-1, LedRect.Bottom-1, 10, 10);
      break;
    case lsRound   :
      Canvas -> Ellipse(LedRect.Left, LedRect.Top, LedRect.Right, LedRect.Bottom);
      break;
    }
  // draw phong when Ctl3D and lsRndRect or lsRound
  if ( ( Ctl3D == true ) && ( FLedShape == lsRndRect ) || ( FLedShape == lsRound ) )
    {
    if ( FLedOn )
      {
      Canvas -> Pen -> Color = clWhite;
      Canvas -> Pen -> Width = 1;
      Canvas -> Brush -> Color = clWhite;
      Canvas -> Brush -> Style = bsSolid;
      }
    else
      {
      Canvas -> Pen -> Color = clSilver;
      Canvas -> Pen -> Width = 1;
      Canvas -> Brush -> Color = clSilver;
      Canvas -> Brush -> Style = bsSolid;
      }
    Canvas -> Ellipse((LedRect.Left+(FLedWidth / 3)), (LedRect.Top+(FLedHeight / 3)),
            (LedRect.Left+(FLedWidth / 3)+FPhongSize), (LedRect.Top+(FLedHeight / 3)+FPhongSize));
    }
  // draw caption
  if ( FCaption != "" )
    {
    Canvas -> Brush -> Color = Color;
    Canvas -> TextOut( CaptionOrigin . x, CaptionOrigin . y, FCaption );
    }
  // draw frame if shape is rectangle
  if ( ( Ctl3D == true ) && ( FLedShape == lsRect ) )
    {
//    if ( InflateRect( &RECT( LedRect ), 1, 1 ) )
      switch ( BorderStyle )
        {
        case bsRaised :
          Extctrls::Frame3D( Canvas, LedRect, clWhite, clGray, FBorderWidth );
          break;
        case bsLowered :
          Extctrls::Frame3D( Canvas, LedRect, clGray, clWhite, FBorderWidth );
          break;
        } // end switch
    }  // end if
}

