//---------------------------------------------------------------------------
#ifndef LedH
#define LedH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
//---------------------------------------------------------------------------

// classes

class TLed : public TGraphicControl
{
public:
  typedef enum { bsRaised, bsLowered } TBrdrStyle;
  typedef enum { lsRect, lsRndRect, lsRound } TLedShape;
  typedef enum { laLeft, laRight, laCenter, laTop, laBottom } TMyAlignment;
  typedef int TInterval;  // 0..MaxInt;
  typedef int TBrdrWidth;  // 1..5;
  typedef int TLedDimension;  // 5..50;
  typedef int TMargin;  // 1..10;
  __fastcall TLed(TComponent* Owner);
  __fastcall ~TLed( void );


private:
  typedef void __fastcall (__closure *TOnOff)( TObject *Sender, bool IsOn );

  TMyAlignment FAlignment;
  TColor FBorderColor;
  TBrdrStyle FBorderStyle;
  TBrdrWidth FBorderWidth;
  TCaption FCaption;
  bool FCtl3D;
  TFont *FFont;
  TInterval FInterval;
  TLedDimension FLedHeight;
  bool FLedOn;
  TLedShape FLedShape;
  TLedDimension FLedWidth;
  TMargin FMargin;
  TColor FOffColor;
  TColor FOnColor;
  TOnOff FOnOff;
  TBrdrWidth FPhongSize;
  TTimer *FTimer;

  TRect LedRect;
  TPoint LedOrigin;
  TPoint CaptionOrigin;
  void __fastcall SetAlignment( TMyAlignment value );
  void __fastcall SetBorderStyle( TBrdrStyle value );
  void __fastcall SetBorderWidth( TBrdrWidth value );
  void __fastcall SetCaption( TCaption value );
  void __fastcall SetColor( int index, TColor value );
  void __fastcall SetCtl3D( bool value );
  void __fastcall SetFont( TFont *value );
  void __fastcall SetInterval( TInterval value );
  void __fastcall SetLedDimension( int index, TLedDimension value );
  void __fastcall SetLedOn(bool value );
  void __fastcall SetLedShape( TLedShape value );
  void __fastcall SetMargin( TMargin value );
  void __fastcall SetPhongSize( TBrdrWidth value );

protected:
  virtual void __fastcall Paint( void );
  virtual void __fastcall Loaded( void );
  void __fastcall TimerTimer( TObject *Sender );

__published:
  __property TMyAlignment Alignment = { read = FAlignment, write = SetAlignment, default = laCenter };
  __property TColor BorderColor = { read = FBorderColor, write = SetColor, index = 3, default = clBlack };
  __property TBrdrStyle BorderStyle = { read = FBorderStyle, write = SetBorderStyle, default = bsRaised };
  __property TBrdrWidth BorderWidth = { read = FBorderWidth, write = SetBorderWidth, default = 2 };
  __property TCaption Caption = { read = FCaption, write = SetCaption };
  __property bool Ctl3D = { read = FCtl3D, write = SetCtl3D, default = true };
  __property TFont *Font = { read = FFont, write = SetFont };
  __property TInterval Interval = { read = FInterval, write = SetInterval, default = 0 };
  __property TLedDimension LedHeight = { read = FLedHeight, write = SetLedDimension, index = 1, default = 18 };
  __property bool LedOn = { read = FLedOn, write = SetLedOn, default = false };
  __property TLedShape LedShape = { read = FLedShape, write = SetLedShape, default = lsRound };
  __property TLedDimension LedWidth = { read = FLedWidth, write = SetLedDimension, index = 2, default = 18 };
  __property TMargin Margin = { read = FMargin, write = SetMargin, default = 3 };
  __property TColor OffColor = { read = FOffColor, write = SetColor, index = 1, default = clMaroon };
  __property TColor OnColor = { read = FOnColor, write = SetColor, index = 2, default = clRed };
  __property TBrdrWidth PhongSize = { read = FPhongSize, write = SetPhongSize, default = 2 };
  __property ShowHint;
  __property TOnOff OnChange = { read = FOnOff, write = FOnOff };
};
//---------------------------------------------------------------------------
#endif
