;  MMX2 optimized routines for SAD of 16*16 macroblocks
;	Copyright (C) Juan J. Sierralta P. <juanjo@atmlab.utfsm.cl>
;
;  dist1_* Original Copyright (C) 2000 Chris Atenasio <chris@crud.net>
;  Enhancements and rest Copyright (C) 2000 Andrew Stevens <as@comlab.ox.ac.uk>

;
;  This program is free software; you can redistribute it and/or
;  modify it under the terms of the GNU General Public License
;  as published by the Free Software Foundation; either version 2
;  of the License, or (at your option) any later version.
;
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with this program; if not, write to the Free Software
;  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
;

global pix_abs16x16_mmx2

; int  pix_abs16x16_mmx(unsigned char *pix1,unsigned char *pix2, int lx, int h);
; esi = p1 (init:		blk1)
; edi = p2 (init:		blk2)
; ecx = rowsleft (init:	 h)
; edx = lx;

; mm0 = distance accumulators (4 words)
; mm1 = distance accumulators (4 words) 
; mm2 = temp 
; mm3 = temp
; mm4 = temp
; mm5 = temp 
; mm6 = temp
; mm7 = temp


align 16
pix_abs16x16_mmx2:
	push				ebp					; save frame pointer
	mov				ebp, esp

	push				ebx					; Saves registers (called saves convention in
	push				ecx					; x86 GCC it seems)
	push				edx					; 
	push				esi
	push				edi
		
	pxor				mm0, mm0				; zero acculumators
	pxor				mm1, mm1
	mov				esi, [ebp+8]		; get pix1
	mov				edi, [ebp+12]		; get pix2
	prefetchnta			[esi]
	prefetchnta			[edi]
	mov				edx, [ebp+16]		; get lx
	mov				ecx, [ebp+20]		; get rowsleft
	jmp				.next4row
align 16

.next4row:
	; First row
	
	prefetchnta			[esi+edx]
	prefetchnta			[edi+edx]
	movq				mm4, [edi]		; load first 8 bytes of pix2 row 
	movq				mm5, [edi+8]	; load last 8 bytes of pix2 row
	psadbw			mm4, [esi]		; SAD of first 8 bytes
	psadbw			mm5, [esi+8]	; SAD of last 8 bytes
	paddw				mm0, mm4			; Add to acumulators
	paddw				mm1, mm5
		
	; Second row	

	add				edi, edx;
	add				esi, edx;
	prefetchnta			[esi+edx]
	prefetchnta			[edi+edx]
	
	movq				mm6, [edi]		; load first 8 bytes of pix2 row 
	movq				mm7, [edi+8]	; load last 8 bytes of pix2 row
	psadbw			mm6, [esi]		; SAD of first 8 bytes
	psadbw			mm7, [esi+8]	; SAD of last 8 bytes
	paddw				mm0, mm6			; Add to acumulators
	paddw				mm1, mm7
		
	; Third row
	
	add				edi, edx;
	add				esi, edx;
	prefetchnta			[esi+edx]
	prefetchnta			[edi+edx]
	
	movq				mm4, [edi]		; load first 8 bytes of pix2 row 
	movq				mm5, [edi+8]	; load last 8 bytes of pix2 row
	psadbw			mm4, [esi]		; SAD of first 8 bytes
	psadbw			mm5, [esi+8]	; SAD of last 8 bytes
	paddw				mm0, mm4			; Add to acumulators
	paddw				mm1, mm5
		
	; Fourth row	
	add				edi, edx;
	add				esi, edx;
	prefetchnta			[esi+edx]
	prefetchnta			[edi+edx]
	
	movq				mm6, [edi]		; load first 8 bytes of pix2 row 
	movq				mm7, [edi+8]	; load last 8 bytes of pix2 row
	psadbw			mm6, [esi]		; SAD of first 8 bytes
	psadbw			mm7, [esi+8]	; SAD of last 8 bytes
	paddw				mm0, mm6			; Add to acumulators
	paddw				mm1, mm7
	
	; Loop termination

	add				esi, edx		; update pointers to next row
	add				edi, edx		
	prefetchnta			[esi+edx]
	prefetchnta			[edi+edx]
	sub				ecx,4
	test				ecx, ecx		; check rowsleft
	jnz				near .next4row
	
	paddd				mm0, mm1		; Sum acumulators
	movd				eax, mm0		; Store return value

	pop edi
	pop esi	
	pop edx			
	pop ecx			
	pop ebx			

	pop ebp							; restore stack pointer

	;emms								; clear mmx registers
	ret								; return


